 
/***************************************************************|
|                   MCU Software Configuration                  |
|               Hardware Abstract Layer (Diall_HAL)             |
|*********************** Config_Diall.h ************************|
|    Suported MCU: PIC24FJ64GB002,PIC24FJ64GA002,PIC24FJ64GB004 |
|    Types of enviroment: MCU Config, Global Types, Debug Mode, |
|                         Threading                             |
|    Written by: Diallix, Diallix Software (C)                  |
|    For more info visit: http:\\www.diallix.net\mcu            |
|    Last revision code: 12.04 2021                             |
|***************************************************************|
|***************************************************************|
|***************** Preprocessor global macros:*****************************************************************************************************************************************************************************|
|    __OnlyGlobalDef__ - if this macro is defined, all global types and methods of this header will be declare in case, that the current MCU is not supporting - use this macro in case, that you include no-suported MCU! |
|    __NoTypeDef__     - if this macro is defined, no global types, of this header, are defined.											                                                                               |
|    _v_clock_         - to define extern delay frequency. If this is defined, default frequency is used.															                                                       |
|    _Clock            - to define extern delay function. If this is not defined, default function is used.		                     																				       |
|    __enable__threading_mode__  - if this macro is defined, Threading mode is enabled. The main fuction is "int main_diall(){..code..}. 																				   |
|                                - if this macro is not defined, Threading mode is disabled. The main function is "int main(){..code..}.										  								    	   |
|    __disble_internal_interrupt_cnf__  - if this macro is defined, you must set external configuration of interrupt: timer1 - timer5.									  									 		       |
|                                       - if this macro is not defined, system use default configuration of interrupts.																								       |
|    __disable_interrupt_internal_cnf__Threadx  - if this macro is defined, you must set external configuration of the Thread interrupt.																		           |						|                                 									                                                                                                                                                    |
***************************************************************************************************************************************************************************************************************************/

#ifdef __NAMESPACE__
	#undef __NAMESPACE__
#endif

#if defined(__PIC24FJ64GB002__) || defined(__PIC24FJ64GA002__) || defined(__PIC24FJ64GB004__)
	#define __NAMESPACE__ 240640024
	#ifdef __PIC24FJ64GB002__
		#include "p24FJ64GB002.h"
	#endif
	#ifdef __PIC24FJ64GA002__
		#include "p24FJ64GA002.h"
	#endif    
	#ifdef __PIC24FJ64GB004__
		#include "p24FJ64GB004.h"
	#endif  
	#if !defined(__24FJ64GB002_H) && !defined(__24FJ64GA002_H) && !defined(__24FJ64GB004_H)
		#error "Header file/s of this MCU does/do not include!"
	#endif
    #define __CONFING_DIALL__ 
#elif !defined(__OnlyTypeDef__)
	#error "Config file does not support this MCU!"
#else
	#warning "Config file does not support this MCU. Only Types and global FCEs were defined!"
#endif

#ifndef __CONFIG_DIALL_SET__
	#define __CONFIG_DIALL_SET__ 0xcbf29ce4
#elif __CONFIG_DIALL_SET__ == 0xcbf29ce4
	#error "This file has been included!"
#endif

#define _CONFIG_DIALL_H_ __CONFIG_DIALL_SET__

#ifndef _STRING_H_
	#include <string.h>
#endif

#ifndef _STDLIB_H_
	#include <stdlib.h>
#endif

#ifndef _STDINT_H_
	#include <stdint.h>
#endif

#ifndef _STDIO_H_
	#include <stdio.h>
#endif

#ifndef __STDDEF_H
	#include <stddef.h>
#endif

#ifndef _TIME_H
	#include <time.h>
#endif

#if !defined(__NoTypeDef__)
	#warning "Global Types and Enums were declared!"

    typedef enum tabBOOLEAN
	{ 
		FALSE = 0, 
		TRUE = 1 
	}BOOL;

    typedef enum tabBIT
	{ 
		CLEAR = 0,
		SET = 1 
	}BIT;

    typedef signed int INT;
    typedef signed char INT8;
    typedef signed short int INT16;
    typedef signed long int INT32;
 
    typedef unsigned int UINT;
    typedef unsigned char UINT8;
    typedef unsigned short int UINT16;
    typedef unsigned long int UINT32;

    typedef unsigned char BYTE;

	#if !defined(__18CXX)
		typedef signed long long INT64;
		typedef unsigned long long UINT64;
	#else
		typedef unsigned short long UINT24;
	#endif
#else
	#warning "Global Types and Enums are disabled!"
#endif

typedef enum tabBOLEAN
{
	false = 0, 
	true = 1
}bool;

#ifndef byte
	typedef unsigned char byte;
#endif

#ifndef uint32
	typedef unsigned long int uint32;
#endif

#ifndef int8_t
	typedef signed char int8_t;
	typedef unsigned char uint8_t;
    #define INT8_MAX 0x7f
    #define NT8_MIN (-INT8_MAX - 1)
    #define UINT8_MAX (__CONCAT(INT8_MAX, U) * 2U + 1U)
    #define UINT8_C (value) ((uint8_t) __CONCAT(value, U))
#endif

#define LOBYTE(w) ((BYTE)(w))
#define HIBYTE(w) ((BYTE)(((UINT)(w) >> 8) & 0xFF))
#define	ALL_BITS_SET (~0)

#ifndef uintptr
	#define uintptr (unsigned int*)
#endif

#define ON 1
#define OFF 0

#define DISABLE 1
#define ENABLE 0

#define nullptr (0)

#if !defined(NULL)
	#define NULL ((void*)0)
#endif

#ifndef null
	#define null NULL
#endif

unsigned char (*nptr) = NULL;

#ifndef NUL
	#define NUL '\0'
#endif

#ifndef nul
	#define nul NUL
#endif

#ifndef False
	#define False 0
#endif

#ifndef True
	#define True 1
#endif

#ifdef RIGHT
	#undef RIGHT
#endif
#define RIGHT 0

#ifdef LEFT
	#undef LEFT
#endif
#define LEFT 1

#ifdef BACKWARD
	#undef BACKWARD
#endif
#define BACKWARD LEFT

#ifdef FORWARD
	#undef FORWARD
#endif
#define FORWARD RIGHT

#ifdef UP
	#undef UP
#endif
#define UP 2

#ifdef DOWN
	#undef DOWN
#endif
#define DOWN 3

typedef enum tabWAYTRACKING
{
	Right = RIGHT,
	Left = LEFT,
	Backward = BACKWARD,
	Forward = FORWARD,
	Up = UP,
	Down = DOWN
}WayTracking;

#define AND \|
#define CUPLE \\
#define END -1
#define NONE -1
#define none NONE

#define HANDLE_SUCCESS 0
#define HANDLE_FAILURE 189

typedef char * string;
typedef unsigned char * STRING;

#define  __No__Used_ -3
int  __No__Used__ = (int)-3;

#define ARRAY_SIZE(array)	(sizeof(array)/sizeof(array[0]))

#if !defined(PROTECTED)
	#define PROTECTED
#endif
#if !defined(PUBLIC)
	#define PUBLIC
#endif
#if !defined(PRIVATE)
	#define PRIVATE static
#endif

#if defined(operator)
	#undef operator
#endif
#define operator

#ifndef _v_clock_
	#define _v_clock_ 8000000
	#warning "MCU external Dealy frequency (mhz) is default set on: 8 MHZ."
#endif

#ifndef _v_clock_
	#define _v_clock_ 8000000
	#warning "MCU external Dealy frequency (mhz) is default set on: 8 MHZ."
#endif

#ifndef _Clock
	#define _Clock(x) ((x - 2500000)/1000000)-(x/2)/1000000
	#warning "MCU external Delay function is default set by fce: (mhz - 2.5) - (mhz / 2)."
#else
	#warning "MCU external Delay function is set by User!"
#endif

#define NOP10(void) asm("nop;nop;nop;nop;nop;nop;nop;nop;nop;nop")
#define NOP1(void) asm("nop")

#define __POLYNOM_32__ 0x23456789
#define __POLYNOM_31__ 0xABCDE

PUBLIC struct tagSYS {
	struct tagGUARTER {
		struct __attribute__ ((packed)) {
			int * No_Used_Pin;
			int * Usb_Pin;
            int * E_Osco;
            int Use_Port;
            int Max_Pin;
            int No_Used_p_size;
            int USB_Pin_size;
            int E_Osco_size;
		}Pins;    

    	struct __attribute__ ((packed)) {
			int Bad_Port;
            int OverFlow;
            int Timing;
		}Error;

		int   Frequency;
		int   Fce;
		char * Name;
		bool  IsSystemInit;
	}Mcu;

    struct __attribute__ ((packed)) {
    	int Led;
        bool Debug;
        volatile unsigned int *Port;
        volatile unsigned int *Tris;
    }Detect;
}This;

#if defined(__PIC24FJ64GB002__) || defined(__24FJ64GB002_H) || defined(__PIC24FJ64GA002__) || defined(__24FJ64GA002_H) || defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)

	#if defined(__PIC24FJ64GA002__) || defined(__24FJ64GA002_H)
		int _No_Used_Pin[] = {__No__Used_,__No__Used_};
		int _USB_Pin[] = {__No__Used_,__No__Used_};
	#endif

	#if defined(__PIC24FJ64GB002__) || defined(__24FJ64GB002_H)
		int _No_Used_Pin[] = {6,12};
		int _USB_Pin[] = {10,11};
	#endif

	#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
		int _No_Used_Pin[] = {5,6,12};
		int _USB_Pin[] = {10,11};
	#endif

	int _E_Osco[] = {1};
	int _Max_pin_ = 15;            
	int _Anl_pins[4][16] = { 
	             				    /* 0 // PORTA */ {0,1,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_},
								#if defined(__PIC24FJ64GA002__) || defined(__24FJ64GA002_H)
	             					/* 1 // PORTB */ {2,3,4,5,6,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,12,11,10,9},  
								#endif
								#if defined(__PIC24FJ64GB002__) || defined(__24FJ64GB002_H)
	             					/* 1 // PORTB */ {2,3,4,5,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,11,10,9},  
								#endif
								#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
	            					/* 1 // PORTB */ {2,3,4,5,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,11,10,9},  
								#endif  
								#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
	             					/* 2 // PORTC */ {6,7,8,12,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_},  
								#else  
	             					/* 2 // PORTC */ {__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_},
								#endif 		     
	            				    /* 3 // PORTD */ {__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_,__No__Used_}
	                      };
                       
	int _No_Used_Pin_size = sizeof(_No_Used_Pin)/sizeof(int);
	int _USB_Pin_size = sizeof(_USB_Pin)/sizeof(int);
	int _E_Osco_size = sizeof(_E_Osco)/sizeof(int);
     
	#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
		#define _Use_Ports 3 // PORTA, PORTB, PORTC
	#else
		#define _Use_Ports 2 // PORTA, PORTB
	#endif

	#if !defined(__PIC24FJ64GB004__) && !defined(__24FJ64GB004_H)
		#define PORTC __No__Used__
		#define TRISC __No__Used__
	#endif
		#define PORTD __No__Used__
		#define TRISD __No__Used__
	
	    #define RA0 LATAbits.LATA0
	    #define RA1 LATAbits.LATA1
	    #define RA2 LATAbits.LATA2
	    #define RA3 LATAbits.LATA3
	    #define RA4 LATAbits.LATA4
	    #define RA0_ TRISAbits.TRISA0
	    #define RA1_ TRISAbits.TRISA1
	    #define RA2_ TRISAbits.TRISA2
	    #define RA3_ TRISAbits.TRISA3
	    #define RA4_ TRISAbits.TRISA4

		#define RB0 LATBbits.LATB0
		#define RB1 LATBbits.LATB1
	    #define RB2 LATBbits.LATB2
	    #define RB3 LATBbits.LATB3
	    #define RB4 LATBbits.LATB4
	    #define RB5 LATBbits.LATB5
	    #define RB7 LATBbits.LATB7
	    #define RB8 LATBbits.LATB8
	    #define RB9 LATBbits.LATB9
	    #define RB10 LATBbits.LATB10
	    #define RB11 LATBbits.LATB11
	    #define RB13 LATBbits.LATB13
	    #define RB14 LATBbits.LATB14
	    #define RB15 LATBbits.LATB15
	    #define RB0_ TRISBbits.TRISB0
	    #define RB1_ TRISBbits.TRISB1
	    #define RB2_ TRISBbits.TRISB2
	    #define RB3_ TRISBbits.TRISB3
	    #define RB4_ TRISBbits.TRISB4
	    #define RB5_ TRISBbits.TRISB5
	    #define RB7_ TRISBbits.TRISB7
	    #define RB8_ TRISBbits.TRISB8
	    #define RB9_ TRISBbits.TRISB9
	    #define RB10_ TRISBbits.TRIS10
	    #define RB11_ TRISBbits.TRISB11
	    #define RB13_ TRISBbits.TRISB13
	    #define RB14_ TRISBbits.TRISB14
	    #define RB15_ TRISBbits.TRISB15
	
	#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
		#define RC0 LATCbits.LATC0
	    #define RC1 LATCbits.LATC1
	    #define RC2 LATCbits.LATC2
	    #define RC3 LATCbits.LATC3
	    #define RC4 LATCbits.LATC4
	    #define RC5 LATCbits.LATC5
	    #define RC6 LATCbits.LATC6
	    #define RC7 LATCits.LATC7
	    #define RC8 LATCbits.LATC8
	    #define RC9 LATCbits.LATA9
	    #define RC0_ TRISCbits.TRISC0
	    #define RC1_ TRISCbits.TRISC1
	    #define RC2_ TRISCbits.TRISC2
	    #define RC3_ TRISCbits.TRISC3
	    #define RC4_ TRISCbits.TRISC4
	    #define RC5_ TRISCbits.TRISC5
	    #define RC6_ TRISCbits.TRISC6
	    #define RC7_ TRISCbits.TRISC7
	    #define RC8_ TRISCbits.TRISC8
	    #define RC9_ TRISCbits.TRISC9
	#endif
	 
	    #define AN0 0
	    #define AN1 1
	    #define AN2 2
	    #define AN3 3
	    #define AN4 4
	    #define AN5 5
	#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
	    #define AN6 6
	#endif
	    #define AN7 7
	    #define AN8 8
	    #define AN9 9
	    #define AN10 10
	    #define AN11 11
	    #define AN12 12
	
	#if defined(__PIC24FJ64GB004__) || defined(__24FJ64GB004_H)
		int AN_pins [13] = {AN0,AN1,AN2,AN3,AN4,AN5,AN6,AN7,AN8,AN9,AN10,AN11,AN12};
	#else
		int AN_pins [13] = {AN0,AN1,AN2,AN3,AN4,AN5,__No__Used_,AN7,AN8,AN9,AN10,AN11,AN12};
	#endif
	int AN_pins_size = sizeof(AN_pins)/sizeof(int);
#endif

#define Bit_Set_(__reg__,__pin__) PORT ## __reg__  |= 1<<__pin__
#define Bit_Clr_(__reg__,__pin__) PORT ## __reg__ &= ~(1<<__pin__)
#define Bit_Toogle_(__reg__,__pin__) PORT ## __reg__ ^= (1<<__pin__)
#define Bit_Set(__reg__,__pin__) __reg__ |= 1<<__pin__
#define Bit_CSet(__reg__,__pin__) __reg__  = 1<<__pin__
#define Bit_Clr(__reg__,__pin__) __reg__ &= ~(1<<__pin__)
#define Bit_Toogle(__reg__,__pin__) __reg__ ^= (1<<__pin__)
#define Config_IN_(__reg__,__pin__) TRIS ## __reg__  |= 1<<__pin__
#define Config_OUT_(__reg__,__pin__) TRIS ## __reg__ &= ~(1<<__pin__)
#define Config_IN(__reg__,__pin__) __reg__ |= 1<<__pin__
#define Config_OUT(__reg__,__pin__) __reg__ &= ~(1<<__pin__)
#define AD_Set_Digital(__pin__) AD1PCFG |= 1<<__pin__
#define AD_Set_Analog(__pin__) AD1PCFG &= ~(1<<__pin__)
#define Set_Pin(__reg__,__pin__,__position__) __reg__ = __pin__<<__position__
#define Set_On(__set_me__) __set_me__ = ON
#define Set_Off(__set_me__) __set_me__ = OFF
#define Write_Pin(__reg__,__pin__)  __reg__ = __pin__ << 7
#define Get_Value(__reg__,__port__) __reg__ & (1 << __port__)
#define Get_Switch(__reg__,__port__) ~Get_Value(__reg__,__port__)
#define ReSwitching(__x__) ~(__x__)
#define Get_Mask_And(__reg__,__pin__) __reg__ & __pin__
#define Get_Mask_Or(__reg__,__pin__) __reg__ | __pin__
#define Get_Mask_Xor(__reg__,__pin__) __reg__ ^ __pin__
#define Set_ADC_port(__pin__) AD1CSSL |= 1 << __pin__; \
                              AD1CHS0bits.CH0SA  = __pin__; 
#define Scan_ADC_port(__pin__) AD1CSSLbits.CSSL |= 1 << __pin__
#define	UNREFERENCED(x) ((void)x)

#ifdef __enable__threading_mode__
	unsigned char seq[10];
	unsigned char thseq;
	unsigned char thseqtemp;
	#define THREAD_START(A) thseq=seq[A]; thseqtemp=0; if(thseq==thseqtemp) {
	#define THREAD_BREAK } thseqtemp++; if(thseq==thseqtemp) {
	#define THREAD_END(B) } seq[B]++; if(seq[B]>thseqtemp) seq[B]=0;
#endif

PUBLIC void ResetMCU(void);
PRIVATE void SetPrescalers(void);
PUBLIC void RPORPort(volatile unsigned int,int,int);
PRIVATE inline int _sda_fc12a_g_sys_2154(uint32*,uint32);
PRIVATE inline int _sda_ad_rnd(void);
#define Shift_It(x,y) _sda_fc12a_g_sys_2154(x,y)
#define Random(void) _sda_ad_rnd(void)
#define Randoming(x,y) x+(_sda_ad_rnd()%y)
#ifdef __enable__threading_mode__
	#ifdef __enable__threading_devimode__
  		PRIVATE int Get_Prescaler(byte);
	#endif
  	PUBLIC extern void ThreadA(void); 
  	PUBLIC extern void ThreadB(void); 
  	PUBLIC extern void ThreadC(void);
  	PUBLIC extern void ThreadD(void);
  	PUBLIC extern void ThreadE(void);
  	PUBLIC extern void ThreadF(void);
  	PUBLIC int main_diall(void);
#else 
  	PUBLIC extern int main(void);
#endif
PUBLIC inline void Delay_us(int);
PUBLIC inline void Delay_ms(int);
PUBLIC inline void Delay_s(int);
PUBLIC void Detect_Satus(int);
PUBLIC extern char * PageSys(void);
PRIVATE byte Booting(byte(),char*(),char*);
PUBLIC inline void Init_Port(unsigned int,int,char*);
PUBLIC void Init(char*(),char*(),char*(),char*(),char*(),char*(),char*(),byte());
PUBLIC inline void * operator new(size_t);
PUBLIC inline void operator delete(void*);

#define __PageSys__

PUBLIC struct tagLOADER {
	struct __attribute__ ((packed)) {
		char* __init__;
		char* __master__;
		char* __boot__;
		char* __slave__;
		char* _mode_done_;
		char* _status_;
		int   _swap_;
	}Load;

	struct __attribute__ ((packed)) {
      /*   frtm   - 0x34 boot device has to boot one time, no more
           expm   - 0x11 boot device was load succesfully 
           jmp    - 0x21 boot device was disload succesfully
           status - 0x1A/0xFF boot device was/was not init succesfully */
		int status;
		byte frtm;
 		byte exp;
		byte jmp;
		char* load;
		int i;
	}Boot;

	struct __attribute__ ((packed)) {
		int i;
		char * _tmp_;
		int isinit;
	}Temp;         
}Loader;

#ifndef __Status_Led__
	#define __Status_Led__
#endif

PUBLIC struct __attribute__ ((packed)) {
	volatile unsigned int *Port;
	volatile unsigned int *Tris;
	int Led;

	unsigned int _s_;
	int _loop_;
	long _x_;
	long _y_;
	long _z_;
	long _u_;

	int ok_status;
	int bad_status;
 	int init_ok;

	struct __attribute__ ((packed)) {
		int time;
	}Tmp;           
}LedStatus;

#ifndef _Random_
	#define _Random_
#endif

struct __attribute__ ((packed)) {
	uint32 lfsr_31_bit;
	uint32 lfsr_32_bit;
	int feedback;

	struct __attribute__ ((packed)) {
			int time;
	}Tmp;     
}Randoming;

#ifdef __enable__threading_mode__
	struct tagINTERRUPTING {
		uint32 Interrupt_Flag;
		uint32 Interrupt_Count;
		int prescaler;
  	};

	PRIVATE struct tagINTERRUPTING Interrupt1;
	PRIVATE struct tagINTERRUPTING Interrupt2;
	PRIVATE struct tagINTERRUPTING Interrupt3;
	PRIVATE struct tagINTERRUPTING Interrupt4;
	PRIVATE struct tagINTERRUPTING Interrupt5;

	#ifdef __enable__threading_devimode__
		PRIVATE struct tagINTERRUPTING Interrupt_settings;
	#endif
#endif

PUBLIC inline void * operator new(size_t size)
{
	void * p = malloc(size);
	return p;
}

PUBLIC inline void operator delete(void * address)
{
	free(address);
	address = NULL;
}

PUBLIC void Init_Port (unsigned int port, int index, char* status)
{
	int _port_ = HANDLE_FAILURE;

	if (port == PORTA && PORTA != __No__Used__ && TRISA != __No__Used__) {_port_ = 0;}
	if (port == PORTB && PORTB != __No__Used__ && TRISB != __No__Used__) {_port_ = 1;}
	if (port == PORTC && PORTC != __No__Used__ && TRISC != __No__Used__) {_port_ = 2;}
	if (port == PORTD && PORTD != __No__Used__ && TRISD != __No__Used__) {_port_ = 3;}

	if(_port_ >= _Use_Ports)
	{
		_port_ = HANDLE_FAILURE; // -> ERROR /Bad Ports/
	}

	if(status == "digital" && _port_ != HANDLE_FAILURE) 
	{
		if(_Anl_pins[_port_][index]!= __No__Used_) {AD_Set_Digital(_Anl_pins[_port_][index]);}
	}

	if(status == "analog" && _port_ != HANDLE_FAILURE) 
	{
		if(_Anl_pins[_port_][index]!= __No__Used_) {AD_Set_Analog(_Anl_pins[_port_][index]);}
	}
}

PRIVATE byte Booting(byte(*_boot_device_)(),char*(*_master_)(), char* _mode_)
{
	Loader.Temp.i = 0;

	if(_mode_ == "init")
	{
		Loader.Boot.status = (*_boot_device_)(Loader.Load.__init__);
	} else
	if(_mode_ == "boot")
	{ 
		Detect_Satus(21);
		while((Loader.Boot.status != 2) && (Loader.Temp.i != 5))
		{
			Loader.Boot.status = (*_boot_device_)(Loader.Load.__boot__);
			Detect_Satus(21);
			Loader.Temp.i++;
		}
		Loader.Temp.i = 0;
		if(Loader.Boot.status == Loader.Boot.exp)
		{
			if(_master_ != null)
			{
				(*_master_)(Loader.Boot.load, Loader.Load.__master__);
			}

			Loader.Boot.frtm = 0x34;
                            
			while(Loader.Boot.status != Loader.Boot.jmp)
			{
				Loader.Boot.status = (*_boot_device_)(Loader.Load.__boot__);
			}

			return Loader.Boot.status;
		}         
	} 
	return Loader.Boot.status;
}

PUBLIC void Init(char*(*_master_)(), char*(*_slave_A)(), char*(*_slave_B)(), char*(*_slave_C)(), char*(*_slave_C1)(), char*(*_slave_C2)(), char*(*_slave_D)(), byte(*_boot_A)())
{
	This.Mcu.IsSystemInit = false;

	Loader.Load._status_ = "0xFF";
    /*byte _status_2 = 0xFF;*/
	Loader.Load._mode_done_ = "done";
	Loader.Load._swap_ = 1;
	Loader.Boot.exp = 0x11;
	Loader.Boot.jmp = 0x21;

	Loader.Load.__master__ = "master";
	Loader.Load.__slave__  = "slave";
	Loader.Load.__init__   = "init";
	Loader.Load.__boot__   = "boot";

	char * _init_ok_  = "Init - ok  boot";
	char * _init_bad_ = "Init - xx  boot";
	char * _pagesys_  = "Page - ok  load";
	char * _device_1  = "DevA - xx  boot";
	char * _device_2  = "DevB - xx  boot";
	char * _device_3  = "DevC - xx  boot";
	char * _device_31 = "DevD - xx  boot";
	char * _device_32 = "DevE - xx  boot";
	char * _device_4  = "DevF - xx  Load";

   	SetPrescalers();

	PageSys();

	if(_master_ != null)
	{
		Loader.Temp._tmp_ = (*_master_)(_pagesys_,Loader.Load.__init__);
		if(Loader.Temp._tmp_ == Loader.Load._status_)
		{
			Detect_Satus(3);
		}
	}

	if(_boot_A != null)
	{
		Booting(*_boot_A,null,Loader.Load.__init__);
	}

	if(_slave_A != null)
	{
		Loader.Temp._tmp_ = (*_slave_A)(Loader.Load.__init__); 
		if( _master_ != null) 
		{
 			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				Loader.Temp._tmp_ = _device_1; Loader.Load._swap_ = 0;
			}
			(*_master_)(Loader.Temp._tmp_,Loader.Load.__master__);
		} else
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
 			{
				for(Loader.Temp.i = 1; Loader.Temp.i<=2; Loader.Temp.i++)
				{
					Detect_Satus(3);
				};
			}
			return;
		}
	}

	if(Loader.Boot.status == 1)
	{
		Booting(*_boot_A,*_master_,Loader.Load.__boot__);
	}

	if(_slave_B != null)
	{
		Loader.Temp._tmp_ = (*_slave_B)(Loader.Load.__init__);
		if( _master_ != null)
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				Loader.Temp._tmp_ = _device_2; Loader.Load._swap_ = 0;
			}
			(*_master_)(Loader.Temp._tmp_,Loader.Load.__master__);
		} else
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				for(Loader.Temp.i = 1; Loader.Temp.i<=3; Loader.Temp.i++)
				{
					Detect_Satus(3);
				};
			}
        return;
		}
	}

    if(Loader.Boot.status == 1 && Loader.Boot.frtm != 0x34)
	{
		Booting(*_boot_A,*_master_,Loader.Load.__boot__);
	}

	if(_slave_C != null)
	{
		Loader.Temp._tmp_ = (*_slave_C)(Loader.Load.__init__);
		if( _master_ != null)
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				Loader.Temp._tmp_ = _device_3; Loader.Load._swap_ = 0;
			}
			(*_master_)(Loader.Temp._tmp_,Loader.Load.__master__);
		} else
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				for(Loader.Temp.i = 1; Loader.Temp.i<=4; Loader.Temp.i++)
				{
					Detect_Satus(3);
				};
			}
			return;
		}
	}

	if(Loader.Boot.status == 1 && Loader.Boot.frtm != 0x34)
	{
		Booting(*_boot_A,*_master_,Loader.Load.__boot__);
	}

	if(_slave_C1 != null)
	{
		Loader.Temp._tmp_ = (*_slave_C1)(Loader.Load.__init__);
		if( _master_ != null)
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				Loader.Temp._tmp_ = _device_31; Loader.Load._swap_ = 0;
			}
            (*_master_)(Loader.Temp._tmp_,Loader.Load.__master__);
        } else
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{  
				for(Loader.Temp.i = 1; Loader.Temp.i<=4; Loader.Temp.i++) 
				{
					Detect_Satus(3);                                   
				}; 
			}
  			return;
         }
     }

	if(Loader.Boot.status == 1 && Loader.Boot.frtm != 0x34)
	{
		Booting(*_boot_A,*_master_,Loader.Load.__boot__);
	}

	if(_slave_C2 != null)
	{
		Loader.Temp._tmp_ = (*_slave_C2)(Loader.Load.__init__);

		if( _master_ != null)
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				Loader.Temp._tmp_ = _device_32; Loader.Load._swap_ = 0;
			}
			(*_master_)(Loader.Temp._tmp_,Loader.Load.__master__);
		} else
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				for(Loader.Temp.i = 1; Loader.Temp.i<=4; Loader.Temp.i++)
				{
					Detect_Satus(3);
				};
			}
			return;
		}
	}

	if(Loader.Boot.status == 1 && Loader.Boot.frtm != 0x34)
	{
		Booting(*_boot_A,*_master_,Loader.Load.__boot__);
	}

	if(_slave_D != null)
	{
		Loader.Temp._tmp_ = (*_slave_D)(Loader.Load.__init__);

		if( _master_ != null)
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				Loader.Temp._tmp_ = _device_4; Loader.Load._swap_ = 0;
			}
			(*_master_)(Loader.Temp._tmp_,Loader.Load.__master__);
		} else
		{
			if(Loader.Temp._tmp_ == Loader.Load._status_)
			{
				for(Loader.Temp.i = 1; Loader.Temp.i<=5; Loader.Temp.i++)
				{
					Detect_Satus(3);
				};
			}
			return;
		}
	}

	if(Loader.Load._swap_ == 1)
	{
		(*_master_)(_init_ok_,Loader.Load.__master__);
		(*_master_)(Loader.Load._mode_done_,null);
	} else
	{
		(*_master_)(_init_bad_,Loader.Load.__master__);
	}

	This.Mcu.IsSystemInit = true;

    return;
}

PUBLIC void Detect_Satus(int _status_)  
{
    static int _port_;

	if(!LedStatus.Tmp.time)
	{
		if (*LedStatus.Port == PORTA && PORTA != __No__Used__ && TRISA != __No__Used__) { LedStatus.Tris = uintptr &TRISA; _port_ = HANDLE_SUCCESS;}
		if (*LedStatus.Port == PORTB && PORTB != __No__Used__ && TRISB != __No__Used__) { LedStatus.Tris = uintptr &TRISB; _port_ = HANDLE_SUCCESS;}
		if (*LedStatus.Port == PORTC && PORTC != __No__Used__ && TRISC != __No__Used__) { LedStatus.Tris = uintptr &TRISC; _port_ = HANDLE_SUCCESS;}
		if (*LedStatus.Port == PORTD && PORTD != __No__Used__ && TRISD != __No__Used__) { LedStatus.Tris = uintptr &TRISD; _port_ = HANDLE_SUCCESS;}

		LedStatus.Tmp.time = 1;
        // 189; // -> ERROR /Bad Ports/
	}
    
	if(_port_ == HANDLE_SUCCESS)
	{
		Init_Port(*LedStatus.Port,LedStatus.Led,"digital");
	    Config_OUT(*LedStatus.Tris,LedStatus.Led); 
	    Bit_Clr(*LedStatus.Port,LedStatus.Led);
	
		if (_status_ > 0 && _status_ < 22) 
		{
			if (_status_ == 1) // ok_status
			{ 
				LedStatus._loop_ = 3;
	            LedStatus._x_ = 20;
	            LedStatus._y_ = 50;
	            LedStatus._z_ = 32;
	            LedStatus._u_ = 0;
			}
	
			if (_status_ == 2) // init_ok
			{ 
				LedStatus._loop_ = 1;
	            LedStatus._x_ = 2000;
	            LedStatus._y_ = 1000;
	            LedStatus._z_ = 500;
	            LedStatus._u_ = 0;
			}
	
			if (_status_ == 3) // bad_status _master_
			{ 
	            LedStatus._loop_ = 1;
	            LedStatus._x_ = 1000;
	            LedStatus._y_ = 400;
	            LedStatus._z_ = 4000;
	            LedStatus._u_ = 0;
			}
	
			if (_status_ == 21) // booting
			{ 
				LedStatus._loop_ = 1;  
	            LedStatus._u_ = 0;
			}
	
			Delay_ms(LedStatus._x_);
	
			for(LedStatus._s_= 0; LedStatus._s_<LedStatus._loop_; LedStatus._s_++) 
			{
				Delay_ms(LedStatus._y_);
				Bit_Set(*LedStatus.Port,LedStatus.Led);
				Delay_ms(LedStatus._z_);
				Bit_Clr(*LedStatus.Port,LedStatus.Led);
				Delay_ms(LedStatus._u_);
			};
		}
	}
}

PRIVATE void SetPrescalers(void)
{
	#if _v_clock_ == 8000000
		CLKDIVbits.CPDIV0 = 1;
		CLKDIVbits.CPDIV1 = 0;
		CLKDIVbits.RCDIV0 = 0;
		CLKDIVbits.RCDIV1 = 0;
		CLKDIVbits.RCDIV2 = 0;
	#endif
}

PUBLIC void ResetMCU(void)
{
	__asm__ volatile ("reset");
}

PUBLIC inline void Delay_us (int __us__)
{
	unsigned i;
	for(i = 0; i <= __us__* _Clock(_v_clock_); i++)
	{
		NOP1();
	}
}

PUBLIC inline void Delay_ms (int __ms__)
{
	unsigned ms_cycl, us_cycl;
	for(ms_cycl = 0; ms_cycl <= __ms__ * _Clock(_v_clock_); ms_cycl++)
	{
		for(us_cycl = 0; us_cycl <= 100; us_cycl++)
		{
			NOP10();
		}
	}
}

PUBLIC inline void Delay_s (int __s__) 
{
	unsigned s_cycl, ms_cycl;
	for(s_cycl = 0; s_cycl <= __s__ * _Clock(_v_clock_); s_cycl++)
	{
		for(ms_cycl = 0; ms_cycl <= 1000; ms_cycl++)
		{
			Delay_ms(1);
		}
	}
}

PRIVATE inline int _sda_fc12a_g_sys_2154(uint32 * lfsr_type,uint32 polynom)
{
	Randoming.feedback = *lfsr_type & 1;
    *lfsr_type >>= 1;
    if (Randoming.feedback == 1)
	{
		*lfsr_type ^= polynom;
	}
    return *lfsr_type;
}

PRIVATE inline int _sda_ad_rnd(void)
{
	if(Randoming.Tmp.time!=1)
	{
		Randoming.Tmp.time=1;
        Randoming.lfsr_31_bit = 0x23456789;
        Randoming.lfsr_32_bit = 0xABCDE;
	}
    Shift_It(&Randoming.lfsr_32_bit, __POLYNOM_32__);
    return(Shift_It(&Randoming.lfsr_32_bit, __POLYNOM_32__)^Shift_It(&Randoming.lfsr_31_bit, __POLYNOM_31__));
}
           
#ifdef __enable__threading_mode__
	#ifdef __enable__threading_devimode__
		#warning "Mode 'Threading Devi mode' is enabled!"
		PRIVATE int Get_Prescaler(byte __x__)    
		{ 
			if(__x__==0b00) Interrupt_settings.prescaler = 1;
			if(__x__==0b01) Interrupt_settings.prescaler = 8;
			if(__x__==0b10) Interrupt_settings.prescaler = 64;
			if(__x__==0b11) Interrupt_settings.prescaler = 256;
			return Interrupt_settings.prescaler;
		}
	#endif

	#ifndef _disable_Thread_A
		PUBLIC void __attribute__((interrupt, no_auto_psv)) _T1Interrupt(void)   
		{
			Interrupt1.Interrupt_Flag = 1;
			#ifdef __enable__threading_devimode__
				Interrupt1.Interrupt_Count++;
				if(_v_clock_/Get_Prescaler(T1CONbits.TCKPS)/PR1==Interrupt1.Interrupt_Count)
				{
					Interrupt1.Interrupt_Count = 0;
				}
			#endif
			if(This.Mcu.IsSystemInit == true)
			{
				ThreadA();
			}
			IFS0bits.T1IF = 0;
			_T1IF = 0;
		}
 	#else
   		#warning "Thread 'A' is DISABLED!"
 	#endif
 
	#ifndef _disable_Thread_B 
		PUBLIC void __attribute__((interrupt, no_auto_psv)) _T2Interrupt(void)   
		{
			Interrupt2.Interrupt_Flag = 1;
			#ifdef __enable__threading_devimode__
	 			Interrupt2.Interrupt_Count++;
				if(_v_clock_/Get_Prescaler(T2CONbits.TCKPS)/PR2==Interrupt2.Interrupt_Count)
				{
					Interrupt2.Interrupt_Count = 0;
				}
			#endif
			if(This.Mcu.IsSystemInit == true)
			{
				ThreadB();
			}
			IFS0bits.T2IF = 0;
			_T2IF = 0; 
		}
 	#else
		#warning "Thread 'B' is DISABLED!"
 	#endif

	#ifndef _disable_Thread_C
		PUBLIC void __attribute__((interrupt, no_auto_psv)) _T3Interrupt(void)   
		{
			Interrupt3.Interrupt_Flag = 1;
			#ifdef __enable__threading_devimode__
				Interrupt3.Interrupt_Count++;
				if(_v_clock_/Get_Prescaler(T3CONbits.TCKPS)/PR3==Interrupt3.Interrupt_Count)
				{
					Interrupt3.Interrupt_Count = 0;
				}
			#endif
			if(This.Mcu.IsSystemInit == true)
			{
				ThreadC();
			}
			IFS0bits.T3IF = 0;
			_T3IF = 0; 
		}
	#else
		#warning "Thread 'C' is DISABLED!"
 	#endif

	#ifndef _disable_Thread_D
		PUBLIC void __attribute__((interrupt, no_auto_psv)) _T4Interrupt(void)   
		{
			Interrupt4.Interrupt_Flag = 1;
			#ifdef __enable__threading_devimode__
				Interrupt4.Interrupt_Count++;
				if(_v_clock_/Get_Prescaler(T4CONbits.TCKPS)/PR4==Interrupt4.Interrupt_Count)
				{
					Interrupt4.Interrupt_Count = 0;
				}
			#endif
			if(This.Mcu.IsSystemInit == true)
			{
				ThreadD();
			}
			IFS1bits.T4IF = 0;
			_T4IF = 0 ;   
		}
	#else
		#warning "Thread 'D' is DISABLED!"
	#endif

	#ifndef _disable_Thread_E
		PUBLIC void __attribute__((interrupt, no_auto_psv)) _T5Interrupt(void) 
		{
			Interrupt5.Interrupt_Flag = 1;
			#ifdef __enable__threading_devimode__
				Interrupt5.Interrupt_Count++;
				if(_v_clock_/Get_Prescaler(T5CONbits.TCKPS)/PR5==Interrupt5.Interrupt_Count)
				{
					Interrupt5.Interrupt_Count = 0;
	 			}
			#endif
			if(This.Mcu.IsSystemInit == true)
			{
				ThreadE();
			}
			IFS1bits.T5IF = 0;
			_T5IF = 0 ;
		}
	#else
		#warning "Thread 'E' is DISABLED!"
	#endif
#endif

#ifdef __enable__threading_mode__
	#warning "Threading mode is defined. Rename ->'int main(void){...}'<- to ->'int main_diall(void){...}'<- and Use it as Current!"
	PUBLIC int main(void)
	{
		UNREFERENCED(new);
		UNREFERENCED(delete);

		T1CONbits.TON = 0;
        T2CONbits.TON = 0;   
        T3CONbits.TON = 0;
        T4CONbits.TON = 0;
        T5CONbits.TON = 0;

		#ifndef __disable_interrupt_internal_cnf__
     		#ifndef _disable_Thread_A
        		#ifndef __disable_interrupt_internal_cnf__ThreadA
		        	T1CONbits.TGATE = 0;
		        	T1CONbits.TCS = 0;
		        	T1CONbits.TCKPS = 0b10;
		        	TMR1 = 0x0000;
		        	PR1 = 80;
		        	IPC0bits.T1IP = 0x06;
		        	IFS0bits.T1IF = 0;
		        	IEC0bits.T1IE = 1;
		        	T1CONbits.TON = 1;
        		#endif
     		#endif
     		#ifndef _disable_Thread_B
        		#ifndef __disable_interrupt_internal_cnf__ThreadB
		        	T2CONbits.T32 = 0;
		        	T2CONbits.TGATE = 0;
		        	T2CONbits.TCS = 0;
		        	T2CONbits.TCKPS = 0b01;
		        	TMR2 = 0x0000;
		        	PR2 = 140;
		        	IPC1bits.T2IP = 0x05;
		        	IFS0bits.T2IF = 0;
		        	IEC0bits.T2IE = 1;
		        	T2CONbits.TON = 1;
        		#endif
     		#endif
     		#ifndef _disable_Thread_C
        		#ifndef __disable_interrupt_internal_cnf__ThreadC
	       			T3CONbits.TGATE = 0;
			        T3CONbits.TCS = 0;
			        T3CONbits.TCKPS = 0b10;
			        TMR3 = 0x0000;
			        PR3 = 460;
			        IPC2bits.T3IP = 0x04;
			        IFS0bits.T3IF = 0;
			        IEC0bits.T3IE = 1;
			        T3CONbits.TON = 1;
        		#endif
     		#endif
      		#ifndef _disable_Thread_D
        		#ifndef __disable_interrupt_internal_cnf__ThreadD
	       			T4CONbits.TGATE = 0;
	       			T4CONbits.TCS = 0;
	       			T4CONbits.TCKPS = 0b11;
	       			TMR4 = 0x0000;
	       			PR4 = 680;
	       			IPC6bits.T4IP = 0x03;
	       			IFS1bits.T4IF = 0;
	       			IEC1bits.T4IE = 1;
	       			T4CONbits.TON = 1;
        		#endif
     		#endif
      		#ifndef _disable_Thread_E
        		#ifndef __disable_interrupt_internal_cnf__ThreadE
	       			T5CONbits.TGATE = 0;
	       			T5CONbits.TCS = 0;
	       			T5CONbits.TCKPS = 0b10;
	      			TMR5 = 0x0000;
	       			PR5 = 460;              
	       			IPC7bits.T5IP = 0x04;
	       			IFS1bits.T5IF = 0;
	       			IEC1bits.T5IE = 1;
	       			T5CONbits.TON = 1;
        		#endif
      		#endif
		#else
       		#warning "Threading mode is defined, but default configuration of interrupts is not defined! Define it explicit."
     	#endif

        Interrupt1.Interrupt_Flag = 0;
        Interrupt2.Interrupt_Flag = 0;
        Interrupt3.Interrupt_Flag = 0;
        Interrupt4.Interrupt_Flag = 0;
        Interrupt5.Interrupt_Flag = 0;

        main_diall();

		return 1;
	}
#else
	#warning "Threading mode is not defined! Use ->'int main(void){...}' as Current!"
#endif

PUBLIC void RPORPort(volatile unsigned int Port, int pin, int value)
{
    if (Port == PORTA && PORTA != __No__Used__ && TRISA != __No__Used__)
    {
		if(pin == 0)
        {
			RPOR2bits.RP5R = value;
		}
		if(pin == 1)
        {
			RPOR3bits.RP6R = value;
		}
    }

    if (Port == PORTB && PORTB != __No__Used__ && TRISB != __No__Used__)
    {
		if(pin == 0)
        {
			RPOR0bits.RP0R = value;
		}
		if(pin == 1)
        {
			RPOR0bits.RP1R = value;
		}
		if(pin == 2)
        {
			RPOR1bits.RP2R = value;
		}
		if(pin == 3)
        {
			RPOR1bits.RP3R = value;
		}
		if(pin == 4)
        {
			RPOR2bits.RP4R = value;
		}
		if(pin == 7)
        {
			RPOR3bits.RP7R = value;
		}
		if(pin == 8)
        {
			RPOR4bits.RP8R = value;
		}
		if(pin == 9)
        {
			RPOR4bits.RP9R = value;
		}
		if(pin == 10)
        {
			RPOR5bits.RP10R = value;
		}
		if(pin == 11)
        {
			RPOR5bits.RP11R = value;
		}
		if(pin == 13)
        {
			RPOR6bits.RP13R = value;
		}
		if(pin == 14)
        {
			RPOR7bits.RP14R = value;
		}
		if(pin == 15)
        {
			RPOR7bits.RP15R = value;
		}
    }

	if (Port == PORTC && PORTC != __No__Used__ && TRISC != __No__Used__) 
    {
    	if(pin == 0)
        {
			RPOR8bits.RP16R = value;
		}
    	if(pin == 1)
        {
			RPOR8bits.RP17R = value;
		}
    	if(pin == 2)
        {
			RPOR9bits.RP18R = value;
		}
    	if(pin == 3)
        {
			RPOR9bits.RP19R = value;
		}
    	if(pin == 4)
        {
			RPOR10bits.RP20R = value;
		}
    	if(pin == 5)
        {
			RPOR10bits.RP21R = value;
		}
    	if(pin == 6)
        {
			RPOR11bits.RP22R = value;
		}
    	if(pin == 7)
        {
			RPOR11bits.RP23R = value;
		}
    	if(pin == 8)
        {
			RPOR12bits.RP24R = value;
		}
    	if(pin == 9)
        {
			RPOR12bits.RP25R = value;
		}
	}
}

#undef Shift_It
#undef __POLYNOM_32__
#undef __POLYNOM_31__

#ifndef _Diall_
	#define _Diall_ 0xac2f01
	#warning "Created by Diallix, Diallix Software (C)"
#endif
